/***************************************************************************
                          Random.h  -  description
                             -------------------
    begin                : Wed Dec 6 2000
    copyright            : (C) 2000 by Christian Blum
    email                : cblum@ulb.ac.be
 ***************************************************************************/

/***************************************************************************
    Program's name: hc-mmas-ubqp

    Ant Colony Optimization algorithm to tackle 
    Unary Binary Quadratic Programming

    Copyright (C) 2003  Christian Blum

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Author's contact details: 

    email: cblum@ulb.ac.be
    mail address: Universite Libre de Bruxelles, IRIDIA,
                  Av. Franklin Roosevelt 50, CP 194/6,
                  B-1050 Brussels, Belgium

 ***************************************************************************/

#ifndef RANDOM_H
#define RANDOM_H

#include "config.h"

#include <algorithm>
#include <vector>
#include <stdlib.h>
#include <math.h>

class Random {
private:
  /* constants for a pseudo-random number generator, 
     for details see numerical recipes in C 
  */
  static const int IA = 16807;
  static const int IM = 2147483647;
  static const double AM = (1.0/IM);
  static const int IQ = 127773;
  static const int IR = 2836;
  
  double ran01(long *idum);
  double gasdev(long *idum);

public:
  Random(const int& arg) : seed(arg) {}
  long int seed;
  double next() { return ran01(&seed);}
  double next_normal() { return gasdev(&seed);}
  /*    
      FUNCTION:      generates a random vector, quick and dirty
      INPUT:         vector dimension
      OUTPUT:        returns pointer to vector, 
                     free memory after using the vector
      (SIDE)EFFECTS: none
  */
  long int* generate_array(const int & size);
  /*
   * alternative
   */
  vector<long int> generate_random_vector(const int& size ) {
    vector<long int> V(size);
    generate(V.begin(), V.end(), rand);
    return V;
  }

};
#endif
